from sandbox import sdk2
from sandbox.sdk2 import svn
from sandbox.sdk2.helpers import subprocess as sp

from sandbox.projects.common.environments import SandboxJavaJdkEnvironment
from sandbox.projects.common.environments import SandboxMavenEnvironment


class SITESEARCH_SUPPORT_BINARY2(sdk2.Resource):
    """
    Sitesearch support jars
    """
    releasers = ['iceflame', 'avhaliullin']
    releasable = True


class BuildSitesearchSupport(sdk2.Task):
    class Requirements(sdk2.Requirements):
        environments = [SandboxJavaJdkEnvironment('1.8.0'), SandboxMavenEnvironment('3.2.2')]

    class Parameters(sdk2.Task.Parameters):
        svn_revision = sdk2.parameters.String('revision', default=None, description='Arcadia revision')

    def on_execute(self):
        clone_path = unicode(self.path('sitesearch'))
        svn.Arcadia.checkout('arcadia:/arc/trunk/arcadia/sitesearch', clone_path, revision=self.Parameters.svn_revision)

        with sdk2.helpers.ProcessLog(self, logger="mvn") as pl:
            process = sp.Popen(
                [
                    'mvn',
                    '-Djava.net.preferIPv4Stack=false',
                    '-Djava.net.preferIPv6Addresses=true',
                    '-s', 'settings.xml',
                    '-f', 'support-assembly',
                    'clean', 'package'
                ],
                cwd=clone_path, stdout=pl.stdout
            )
            process.timeout = 10800
            process.wait()

        resource = SITESEARCH_SUPPORT_BINARY2(self, "Sitesearch support jars", unicode(self.path('sitesearch/support-assembly/target/assembly/lib/')))
        sdk2.ResourceData(resource).ready()
