# -*- coding: utf-8 -*-
import os

from sandbox import sdk2
import time
import sandbox.common.types.resource as ctr
import sandbox.projects.resource_types as rt

from sandbox.sdk2.helpers import subprocess as sp
from sandbox.projects.bsyeti.common import dump_timestamp, basic_releasers


class MakeCaesarTurboOriginalUrlsBaseBin(rt.ARCADIA_PROJECT):
    """
    resource with caesar make_turbo_original_urls_base bin
    """

    releasable = True
    any_arch = True
    auto_backup = True
    releasers = basic_releasers + ["adygamovilyas"]
    release_subscribers = ["bulatman", "bsyeti-watcher"]


class CaesarTurboOriginalUrlsBase(sdk2.Resource):
    """
    turbo_original_urls base for caesar
    """

    any_arch = True
    auto_backup = True
    calc_md5 = True
    share = True
    releasable = True
    releasers = basic_releasers + ["adygamovilyas"]


class MakeCaesarTurboOriginalUrlsBase(sdk2.Task):
    """
    task to build turbo_original_urls base for caesar
    """
    class Requirements(sdk2.Requirements):
        cores = 1  # exactly 1 core
        ram = 1024 * 25  # 25GiB or less

    class Caches(sdk2.Requirements.Caches):
        pass  # means that task do not use any shared caches

    class Parameters(sdk2.Parameters):
        binary_id = sdk2.parameters.LastReleasedResource(
            'caesar make turbo original urls base binary resource',
            resource_type=MakeCaesarTurboOriginalUrlsBaseBin,
            state=(ctr.State.READY, ),
            required=True
        )
        yt_proxy = sdk2.parameters.String('Yt proxy')
        source_table = sdk2.parameters.String('Source table')
        do_release = sdk2.parameters.Bool(
            "Make this resourse released (USE ONLY FOR SCHEDULERS!)",
            default=False,
            required=False,
            do_not_copy=True
        )

    def on_execute(self):
        bin_res = sdk2.ResourceData(self.Parameters.binary_id)

        folder = "caesar-turbo-original-urls-base"
        env = {
            "YT_TOKEN": sdk2.Vault.data("sec-01djab78jh9dvsymnjzwhk1mpf[yt_token]"),  # "zomb-yeti" secret from yav
            "YQL_TOKEN": sdk2.Vault.data("sec-01djab78jh9dvsymnjzwhk1mpf[yql_token]"),  # "zomb-yeti" secret from yav
            "TMP": ".",
            "TMPDIR": ".",
        }
        cmd = [
            str(bin_res.path),
            "--proxy", self.Parameters.yt_proxy,
            "--source-table", self.Parameters.source_table,
            "--output-folder", folder,
        ]
        t = time.time()
        os.mkdir(folder)
        with sdk2.helpers.ProcessLog(self, logger="make_caesar_turbo_original_urls_base") as l:
            sp.check_call(cmd, stdout=l.stdout, stderr=l.stderr, env=env)
        dump_timestamp(folder, t)
        res = CaesarTurboOriginalUrlsBase(self, "Caesar Turbo Orignal Urls Base", folder, ttl=14)
        if self.Parameters.do_release:
            res.released = 'stable'
