import logging
import time
import traceback
import uuid

import requests

from sandbox import sdk2


def find_build(token, trigger_id):
    """
    Find build by trigger_id
    """
    deadline = time.time() + 1800
    headers = {'Authorization': 'OAuth {token}'.format(token=token)}
    while time.time() < deadline:
        try:
            builds = requests.get(
                'https://jenkins.db.yandex-team.ru/job/graphene-gen-jobs/api/json', headers=headers).json()['builds']
            for build in builds:
                build_url = '{url}api/json'.format(url=build['url'])
                build_info = requests.get(build_url, headers=headers).json()
                for action in build_info['actions']:
                    if action['_class'] == 'hudson.model.ParametersAction':
                        for parameter in action['parameters']:
                            if parameter['name'] == 'trigger_id' and parameter['value'] == trigger_id:
                                return '{url}api/json'.format(url=build['url'])
        except Exception:
            logging.error('Error on find build: %s', traceback.format_exc())
        time.sleep(5)

    raise RuntimeError('Unable to find running build within timeout (30 minutes)')


def poll_build(token, build_url):
    """
    Poll jenkins build until terminate state
    """
    deadline = time.time() + 10800
    headers = {'Authorization': 'OAuth {token}'.format(token=token)}
    while time.time() < deadline:
        try:
            build_info = requests.get(build_url, headers=headers).json()
            if build_info['result'] is not None:
                if build_info['result'] == 'SUCCESS':
                    return True
                return False
        except Exception:
            logging.error('Error on poll build: %s', traceback.format_exc())
        time.sleep(5)

    raise RuntimeError('Poll build timeout (3 hours)')


class MdbJenkinsSyncGrapheneJobs(sdk2.Task):
    class Requirements(sdk2.Requirements):
        cores = 1
        ram = 1024

        class Caches(sdk2.Requirements.Caches):
            pass

    def on_execute(self):
        token = sdk2.Vault.data('MDB', 'mdb-ci-token')

        trigger_id = str(uuid.uuid4())

        params = {'token': 'token', 'trigger_id': trigger_id}
        res = requests.get(
            'https://jenkins.db.yandex-team.ru/job/graphene-gen-jobs/buildWithParameters',
            params=params,
            headers={'Authorization': 'OAuth {token}'.format(token=token)})
        res.raise_for_status()

        build = find_build(token, trigger_id)

        assert poll_build(token, build), 'Jenkins build failed: {build}'.format(
            build=build.replace('api/json', 'console'))
