from sandbox import sdk2

from sandbox.projects import resource_types

import sandbox.common.types.client as ctc

import StringIO
import requests


class GetResponsesBase(sdk2.Task):
    """ Base task for fetching service responses """

    class DefaultRequirements(sdk2.Task.Requirements):
        client_tags = ctc.Tag.GENERIC & ctc.Tag.Group.LINUX

    class DefaultParameters(sdk2.Task.Parameters):
        port = sdk2.parameters.Integer("Port", default=15358, required=True)
        request_count = sdk2.parameters.Integer("Count of requests", default=1000, required=True)
        experiment = sdk2.parameters.String("Experiment name", default="", required=False)
        queries_resource = sdk2.parameters.Resource(
            "Resource with queries",
            resource_type=resource_types.PLAIN_TEXT_QUERIES,
            required=True
        )

    def parse_next_request(self, f):
        request_size = f.readline()
        if request_size == '':
            return None

        request = f.read(int(request_size))
        f.read(2) # \r\n

        stream = StringIO.StringIO(request)
        first_line = stream.readline().split(' ')
        method = first_line[0]
        url = first_line[1]

        headers = {}
        while True:
            header = stream.readline()[:-2] # \r\n
            if header == '':
                break
            key, value = header.split(':')
            headers[key] = value[1:]

        body = stream.read()

        return (method, url, headers, body)

    def send_request(self, method, url, headers, body):
        full_url = "http://localhost:" + str(self.Parameters.port) + url
        return requests.request(method=method, url=full_url, headers=headers, data=body)

    def __execute__(self, service, responses_resource_data):
        queries_resource_data = sdk2.ResourceData(self.Parameters.queries_resource)

        with open(str(queries_resource_data.path), 'r') as queries_file, open(str(responses_resource_data.path), 'w') as responses_file:
            for i in range(self.Parameters.request_count):
                next_request = self.parse_next_request(queries_file)
                if next_request is None:
                    break
                method, url, headers, body = next_request
                if len(self.Parameters.experiment) > 0:
                    url += "&exp_name="
                    url += self.Parameters.experiment

                response = self.send_request(method, url, headers, body)
                if not response.ok:
                    raise RuntimeError("Error in request processing. Url: {}. Request number: {}".format(url, i))

                responses_file.write(response.content)
                responses_file.write('\n')

        responses_resource_data.ready()

        service.close()
