# coding: utf-8

import os
import sandbox.projects.common.constants as consts

from sandbox import sdk2
from sandbox.projects.common.build.ya_package_config import consts as ya_package_consts
from sandbox.projects.common.build.KosherYaPackage import KosherYaPackage
from sandbox.projects.common.nanny import nanny
from sandbox.sandboxsdk.svn import Arcadia
import sandbox.projects.common.build.parameters as build_params
from sandbox.projects.images.quickdaemon.resources import IMAGES_QUICKDAEMON_LOOP_CONFIG


class ImagesBuildQuickPackage(KosherYaPackage, nanny.ReleaseToNannyTask2):
    class Requirements(KosherYaPackage.Requirements):
        pass

    class Parameters(KosherYaPackage.Parameters):
        resource_type = build_params.ResourceType(default_value='IMAGES_QUICK_PACKAGE')
        packages = build_params.Packages(default_value='extsearch/images/robot/packages/quick.json')
        build_type = build_params.BuildType(default_value=consts.RELEASE_BUILD_TYPE)
        publish_package = build_params.PublishPackage(default_value=False)
        raw_package = build_params.RawPackage(default_value=False)
        package_type = build_params.PackageType(default_value=ya_package_consts.PackageType.TARBALL.value)
        do_not_remove_resources = build_params.DoNotRemoveResources2(default_value=False)
        strip_binaries = build_params.StripBinaries2(default_value=True)

    def on_execute(self):
        self.__create_loop_conf()
        super(ImagesBuildQuickPackage, self).on_execute()

    def on_enqueue(self):
        super(ImagesBuildQuickPackage, self).on_enqueue()

    def on_release(self, additional_parameters):
        nanny.ReleaseToNannyTask2.on_release(self, additional_parameters)
        super(ImagesBuildQuickPackage, self).on_release(additional_parameters)

    def __create_loop_conf(self):
        resource = IMAGES_QUICKDAEMON_LOOP_CONFIG(self, "Images quickdaemon loop.conf", 'loop.conf')
        arcadia_url_full = self.Parameters.checkout_arcadia_from_url.split('@')
        arcadia_root = arcadia_url_full[0]
        arcadia_revision = arcadia_url_full[1] if len(arcadia_url_full) > 1 else None
        arcadia_path = IMAGES_QUICKDAEMON_LOOP_CONFIG.arcadia_path
        arcadia_url = os.path.join(arcadia_root, arcadia_path)
        Arcadia.export(arcadia_url, resource.path, arcadia_revision)
        data = sdk2.ResourceData(resource)
        data.ready()
