# coding: utf-8
import os

import sandbox.common.types.misc as ctm
from sandbox.projects.common import utils
from sandbox.projects.common.nanny import nanny
from sandbox.projects.nanny.common.build_helpers import git_clone_and_checkout
from sandbox.projects.nanny.common.gobabygo_parameters import GitRefIdParameter, GitRefShaParameter, ReleaseParameter
from sandbox.projects.nanny.common.environments import GoDistEnvironment
from sandbox.projects.resource_types import nanny_releasers, AbstractResource
from sandbox.sandboxsdk.process import run_process
from sandbox.sandboxsdk.task import SandboxTask


class HASHICORP_VAULT_BINARY(AbstractResource):
    """
        BuildHashicorpVault result
    """
    releasable = True
    any_arch = False
    executable = True
    auto_backup = True
    releasers = nanny_releasers


class BuildHashicorpVault(nanny.ReleaseToNannyTask, SandboxTask):
    type = 'BUILD_HASHICORP_VAULT'
    dns = ctm.DnsType.DNS64
    GIT_URL = 'https://{username}:{password}@bitbucket.yandex-team.ru/scm/Nanny/vault-blackbox'

    input_parameters = [GitRefIdParameter, GitRefShaParameter, ReleaseParameter]
    environment = (
        GoDistEnvironment('1.7.3'),
    )

    def _get_clone_url(self):
        oauth_token = self.get_vault_data('GBG', 'nanny_robot_bb_oauth_token')
        return self.GIT_URL.format(username='x-oauth-token', password=oauth_token)

    def on_execute(self):
        ref_id = utils.get_or_default(self.ctx, GitRefIdParameter)
        ref_sha = utils.get_or_default(self.ctx, GitRefShaParameter)
        release = utils.get_or_default(self.ctx, ReleaseParameter)
        package_path = os.path.join(os.environ['GOPATH'], 'src/github.com/hashicorp/vault')
        clone_url = self._get_clone_url()

        git_clone_and_checkout(clone_url, package_path, ref_sha or ref_id)

        if release:
            run_process(['make', 'bootstrap'], work_dir=package_path, log_prefix='make bootstrap', shell=True)
            run_process(['make', 'dev'], work_dir=package_path, log_prefix='make bin', shell=True)
            binary_path = os.path.join(os.environ['GOPATH'], 'bin/vault')
            run_process(['cp', binary_path, self.path('vault')], log_prefix='cp', shell=True)
            self.create_resource(
                description='Vault {}'.format(ref_id or ref_sha),
                resource_path=self.path('vault'),
                resource_type=HASHICORP_VAULT_BINARY,
                arch='linux'
            )


__Task__ = BuildHashicorpVault
