# coding: utf8
from __future__ import absolute_import, division, print_function, unicode_literals

import logging
import os

from sandbox import sdk2
from sandbox.projects.common import binary_task
from sandbox.projects.rasp.utils.email_notifications import EmailNotificationMixin, use_email_notification_params
from sandbox.projects.rasp.utils.geobase import GeobaseMixin, use_geobase_params
from sandbox.projects.rasp.utils.juggler import JugglerNotificationMixin, use_juggler_notification_params
from sandbox.projects.rasp.utils.rasp_scripts import RaspScriptsTask, use_rasp_scripts_params

logger = logging.getLogger()
logger.setLevel(logging.INFO)


class RaspGenerateArchivalDataTask(
    binary_task.LastBinaryTaskRelease,
    sdk2.Task,
    EmailNotificationMixin,
    JugglerNotificationMixin,
    RaspScriptsTask,
    GeobaseMixin
):
    class Requirements(sdk2.Task.Requirements):
        cores = 10
        ram = 20 * 1024

        class Caches(sdk2.Requirements.Caches):  # enable multislot https://wiki.yandex-team.ru/sandbox/clients/#client-tags-multislot
            pass  # means that task do not use any shared caches

    class Parameters(sdk2.Task.Parameters):
        kill_timeout = 3600 * 5

        pool_size = sdk2.parameters.Integer('Pool size', required=True, default=5)

        ext_params = binary_task.binary_release_parameters(stable=True)
        _rasp_scripts_params = use_rasp_scripts_params()
        _email_notification_params = use_email_notification_params()
        _juggler_notification_params = use_juggler_notification_params()
        _geobase_params = use_geobase_params()

    def set_task_environ(self):
        os.environ['RASP_ARCHIVAL_POOL_SIZE'] = str(self.Parameters.pool_size)
        os.environ['RASP_GEOBASE_DATA_PATH'] = self.geobase_path

    def on_save(self):
        binary_task.LastBinaryTaskRelease.on_save(self)
        self.add_email_notifications()
        self.add_juggler_notifications(environment=self.Parameters.env_type)

    def on_execute(self):
        binary_task.LastBinaryTaskRelease.on_execute(self)

        self.download_geobase()
        self.set_common_environ()

        from travel.rasp.rasp_scripts.scripts.generate_archival_data import run
        run()
