# coding: utf8
from __future__ import absolute_import, division, print_function, unicode_literals

import pytz
from datetime import datetime

from sandbox import sdk2
from sandbox.projects.common import binary_task
from sandbox.projects.rasp.utils.email_notifications import EmailNotificationMixin, use_email_notification_params
from sandbox.projects.rasp.utils.juggler import JugglerNotificationMixin, use_juggler_notification_params
from sandbox.projects.rasp.utils.rasp_scripts import RaspScriptsTask, use_rasp_scripts_params
from sandbox.projects.rasp.utils.solomon import SolomonProgressMixin


class RaspMinPrices(
    binary_task.LastBinaryTaskRelease,
    sdk2.Task,
    EmailNotificationMixin,
    JugglerNotificationMixin,
    RaspScriptsTask,
    SolomonProgressMixin
):
    class Requirements(sdk2.Task.Requirements):
        cores = 1
        ram = 1 * 1024

        class Caches(sdk2.Requirements.Caches):  # enable multislot https://wiki.yandex-team.ru/sandbox/clients/#client-tags-multislot
            pass  # means that task do not use any shared caches

    class Parameters(sdk2.Task.Parameters):
        kill_timeout = 3600 * 3

        ext_params = binary_task.binary_release_parameters(stable=True)

        _rasp_scripts_params = use_rasp_scripts_params()
        _email_notification_params = use_email_notification_params()
        _juggler_notification_params = use_juggler_notification_params()

    def on_save(self):
        binary_task.LastBinaryTaskRelease.on_save(self)
        self.add_email_notifications()
        self.add_juggler_notifications(environment=self.Parameters.env_type)

    def on_execute(self):
        self.set_common_environ()

        with self.report_progress(source='min_prices', environment=self.Parameters.env_type):
            now = datetime.now(pytz.timezone('Europe/Moscow'))
            if now.hour > 7:
                self.Parameters.tags += ['DAY_RUN']
                from travel.rasp.tasks.min_prices import every_day_run
                every_day_run.run()
            else:
                self.Parameters.tags += ['NIGHT_RUN']
                from travel.rasp.tasks.min_prices import every_night_run
                every_night_run.run()
