# coding: utf8
from __future__ import absolute_import, division, print_function, unicode_literals

import sandbox.projects.release_machine.core.const as rm_const

from sandbox.projects.release_machine.components import configs
from sandbox.projects.release_machine.components.configs.avia import _base_docker_to_deploy
from sandbox.projects.release_machine.core import releasable_items as ri

PROJECT_PATHS = ['arcadia/travel/avia/flight_status_fetcher']
LIMIT_DEPENDENCIES_TO = ['arcadia/travel']
PROJECT_PACKAGE_PATH = 'travel/avia/flight_status_fetcher/pkg.json'

APPLICATION_RESOURCE_NAME = 'AVIA_FLIGHT_STATUS_FETCHER_BINARY'

DEPLOY_PROJECT = 'avia-flight-status-fetcher'

RESPONSIBLE = 'mikhailche'
DEVELOPERS = ['mikhailche']
MANAGERS = ['kurzhumov']
TESTERS = ['gouken67']

ST_QUEUE = 'RASPTICKETS'
ST_COMPONENT = '$flight-status-fetcher'
ST_RELEASE_SUMMARY_TEMPLATE = 'Релиз flight-status-fetcher {}'

WIKI_PAGE = 'avia/dev/services-table/flight-status-fetcher/releases'


class FlightStatusFetcherCfg(configs.ReferenceBranchedConfig):
    display_name = 'Avia flight status fetcher'
    name = 'avia_flight_status_fetcher'
    responsible = RESPONSIBLE

    class Testenv(configs.ReferenceBranchedConfig.Testenv):
        trunk_task_owner = 'AVIA'

        class JobGraph(
            _base_docker_to_deploy.AviaDockerToDeployJobGraphMixin, configs.ReferenceBranchedConfig.Testenv.JobGraph
        ):
            avia_build_package = PROJECT_PACKAGE_PATH
            avia_project_observed_paths = PROJECT_PATHS
            avia_resource_name = APPLICATION_RESOURCE_NAME
            avia_trunk_auto_deploy_stages = []
            avia_release_button_deploy_stages = [
                rm_const.ReleaseStatus.testing,
                rm_const.ReleaseStatus.stable,
            ]

    class Releases(configs.ReferenceBranchedConfig.Releases):
        @property
        def releasable_items(self):
            return [
                ri.ReleasableItem(
                    name=APPLICATION_RESOURCE_NAME,
                    data=_base_docker_to_deploy.get_sandbox_resource_data(APPLICATION_RESOURCE_NAME),
                    deploy_infos=[
                        ri.YaDeployInfo(
                            ri.DeployService(
                                _base_docker_to_deploy.get_deploy_stage_name(
                                    rm_const.ReleaseStatus.stable, DEPLOY_PROJECT
                                )
                            )
                        )
                    ],
                ),
            ]

        allow_robots_to_release_stable = True
        release_followers_permanent = DEVELOPERS + MANAGERS + TESTERS
        wait_for_deploy_time_sec = 5 * 60  # 5 min
        deploy_system = rm_const.DeploySystem.ya_deploy

    class Notify(configs.ReferenceBranchedConfig.Notify):
        class Startrek(configs.ReferenceBranchedConfig.Notify.Startrek):
            """Startrek notifications configuration"""
            assignee = RESPONSIBLE
            queue = ST_QUEUE
            summary_template = ST_RELEASE_SUMMARY_TEMPLATE
            components = ST_COMPONENT
            followers = DEVELOPERS + MANAGERS + TESTERS
            dev_queue = ST_QUEUE

            hide_commits_under_cut = True

    class ChangelogCfg(configs.ReferenceBranchedConfig.ChangelogCfg):
        ya_make_targets = PROJECT_PATHS
        observed_paths = PROJECT_PATHS
        wiki_page = WIKI_PAGE
        add_to_release_notes = True
        svn_paths_filter = configs.ChangelogPathsFilter(rm_const.PermissionType.ALLOWED, LIMIT_DEPENDENCIES_TO)
