# -*- coding: utf-8 -*-

from sandbox.projects.release_machine.core import releasable_items as ri
from sandbox.projects.release_machine.components import configs as cfg
import sandbox.projects.plus.rm as rm_conf
import sandbox.projects.release_machine.core.const as rm_const
from sandbox.projects.release_machine.components.config_core.jg.preset import basic_build_presets
from sandbox.projects.release_machine.components.config_core.jg.cube.lib import build as build_cubes
from sandbox.projects.release_machine.components.config_core.jg.cube.lib import release as release_cubes
from sandbox.projects.release_machine.components.config_core.jg.cube.lib import dummy as dummy_cubes
from sandbox.projects.release_machine.components.config_core.jg.cube import base as jg_cube
from sandbox.projects.release_machine.components.config_core.jg.cube.lib import internal as internal_cubes
from sandbox.projects.release_machine.components.config_core.jg import flow as jg_flow


class PlusMcBackendCfg(cfg.ReferenceCIConfig):

    name = "plus_mc_backend"
    display_name = "Plus Mc Backend"
    responsible = cfg.Responsible(abc=cfg.Abc(component_id=34874, schedule_slug="duty"), login="krabishe")
    component_group = rm_conf.COMPONENT_GROUP

    class CI(cfg.ReferenceCIConfig.CI):
        secret = "sec-01desry8fbgvnkbeybem81ferv"
        sb_owner_group = rm_conf.TRUNK_TASK_OWNER
        a_yaml_dir = "plus/mission_control"
        ya_make_abs_paths_glob = [
            "plus/mission_control/**",
        ]
        grant_config_update_permissions_to_release_machine_robot = False

    class JG(basic_build_presets.SingleBuildGeneralJGCfg):

        @jg_flow.release_flow(stages=basic_build_presets.DEFAULT_RELEASE_FLOW_STAGES)
        def release(self):
            graph = super(PlusMcBackendCfg.JG, self).release(self)
            release_entry = graph.get("release_stage_entry")
            release_stable = graph.get("release_stable_sandbox")
            release_prestable = graph.get("release_prestable_sandbox")
            create_ticket = graph.get(internal_cubes.CreateStartrekTicket.NAME)
            main_entry = graph.get(dummy_cubes.RMMainGraphEntry.NAME)
            build_cube = graph.get("build_afisha_build_docker")

            target_platform_flags = [
                "--target-platform default-linux-x86_64",
                "--target-platform-build-type release",
                "--target-platform-flag ALLOCATOR=LF",
                "--target-platform-flag AUTOCHECK=yes",
                "--target-platform-flag DEBUGINFO_LINES_ONLY=yes",
                "--target-platform-flag NO_DEBUGINFO=no",
                "--target-platform-flag NO_SVN_DEPENDS=yes",
                "--target-platform-flag USE_EAT_MY_DATA=yes"
            ]
            test_params = {
                "checkout_arcadia_from_url": "arcadia-arc:/#${tasks.new_tag.rm_events[2].new_tag_data.arcadia_path}",
                "targets": "plus/mission_control/tests/large-tests",
                "ya_yt_token_vault_owner": "krabishe.yt-token",
                "tests_retries": 2,
                "report_tests_only": True,
                "junit_report": True,
                "test_tag": "ya:fat",
                "test_filters": "plus/mission_control/tests/large-tests",
                "test": True,
                "ya_timeout": 14400,
                "target_platform_flags": ' '.join(target_platform_flags),
                "keep_on": True,
                "clear_build": False,
                "checkout_mode": "auto",
                "build_system": "semi_distbuild",
                "build_output_html_ttl": 14,
                "build_output_ttl": 14,
                "disable_test_timeout": False,
                "cache_test_results": False,
                "check_return_code": True,
                "failed_tests_cause_error": True,
                "test_type_filter": "",
            }

            client_tags = "(GENERIC | OSX | PORTOD) & (~LINUX | LINUX_PRECISE | LINUX_TRUSTY | LINUX_XENIAL | LINUX_BIONIC)"

            regress_tests_cube = jg_cube.Cube(
                name="run_large_tests",
                title="Run Large Regress Tests",
                needs=[main_entry],
                task="common/arcadia/ya_make",
                input=jg_cube.CubeInput(**test_params),
                attributes={
                    "requirements": {
                        "sandbox": {
                            "dns": "dns64",
                            "platform": "linux_ubuntu_16.04_xenial",
                            "client_tags": client_tags,
                            "container_resource": "2686891139",
                        },
                        "ram": "8GB",
                        "disk": "100GB",
                    },
                },
            )

            allow_test_cube = jg_cube.Cube(
                task="dummy",
                title="Allow Large Tests",
                needs=[main_entry, regress_tests_cube],
                needs_type=jg_cube.CubeNeedsType.ANY,
            )

            release_in_build = []
            deploy_system = "sandbox"
            for where in ["testing"]:
                release_task = release_cubes.AfishaRelease2(
                    name="release_{}_{}".format(where, deploy_system),
                    component_name=self.component_name,
                    where_to_release=where,
                    manual=(True if where in ["stable", "prestable"] else False),
                )
                release_task.TYPE = "build"
                release_in_build.append(
                    release_task
                )

            release_to_stable = jg_cube.Cube(
                name="release_to_stable_dummy",
                task="dummy",
                title="Release to stable stage",
                needs=release_in_build,
                manual=True,
            )

            for release_cube in release_in_build:
                release_cube.add_requirement(build_cube)
                graph.add(release_cube)
            release_entry.add_requirement(release_to_stable)
            create_ticket.add_requirement(allow_test_cube)
            release_prestable.add_requirement(release_entry)
            release_stable.add_requirement(release_prestable)

            graph.add(regress_tests_cube)
            graph.add(allow_test_cube)
            graph.add(release_to_stable)
            return graph

        @property
        def release_task(self):
            return rm_conf.TASK_CI_REG_LOCATIONS["release2"]

        @property
        def build_task(self):
            return rm_conf.TASK_CI_REG_LOCATIONS["build"]

        def _get_build_cube(self, graph):
            return build_cubes.AfishaBuildDocker(
                component_name=self.component_name,
                package_path=self.root_cfg.releases_cfg.releasable_items[0].build_data.target,
                needs=[graph.get("main_graph_entry"), graph.get(internal_cubes.CreateStartrekTicket.NAME)],
                input=jg_cube.CubeInput(
                    arc_svn_url="arcadia-arc:/#${tasks.new_tag.rm_events[2].new_tag_data.arcadia_path}",
                    checkout_arcadia_from_url="arcadia-arc:/#${tasks.new_tag.rm_events[2].new_tag_data.arcadia_path}",
                ),
            )

        def _get_release_cubes(self, graph, build):
            result = []
            deploy_system = "sandbox"

            for where in ["stable", "prestable"]:
                result.append(
                    release_cubes.AfishaRelease2(
                        name="release_{}_{}".format(where, deploy_system),
                        component_name=self.component_name,
                        where_to_release=where,
                        manual=(True if where in ["stable", "prestable"] else False),
                    )
                )

            return result

    class Notify(cfg.ReferenceCIConfig.Notify):
        """ Notification settings """

        class Telegram(cfg.ReferenceCIConfig.Notify.Telegram):
            """ Telegram notifications """
            chats = ["mir-releases"]
            invite_link = "https://t.me/joinchat/AoGM6x0jo4CBCpnfIxOgyQ"

        class Startrek(cfg.ReferenceCIConfig.Notify.Startrek):
            """
            Startrek notifications
            https://wiki.yandex-team.ru/releasemachine/startrekoptions/
            """
            assignee = cfg.Responsible(abc=cfg.Abc(component_id=34874, schedule_slug="duty"), login="krabishe")
            queue = rm_conf.STARTREK_QUEUE
            followers = rm_conf.STARTREK_FOLLOWERS
            summary_template = "Plus MC Backend {}"
            add_commiters_as_followers = True
            ticket_type = u"Релиз"
            use_task_author_as_assignee = False
            hide_commits_under_cut = True
            nanny_reports = False
            deadline = 7
            important_changes_limit = rm_conf.IMPORTANT_CHANGES_LIMIT

            @property
            def tags(self):
                return ["rm_main_ticket", self.name, "component_name_{}".format(self.name)]

    class ChangelogCfg(cfg.ReferenceCIConfig.ChangelogCfg):
        """
        Changelog configuration
        https://wiki.yandex-team.ru/releasemachine/changelogs/
        """
        dirs = [
            "arcadia/plus/mission_control",
            "arcadia/plus/user_actions"
        ]
        wiki_page = ""

    class SvnCfg(cfg.ReferenceCIConfig.SvnCfg):

        use_arc = True
        use_zipatch_only = False
        branch_name = "plus/mc_backend"
        tag_name = "plus/mc_backend"
        tag_folder_template = "{branch_num}.{tag_num}"
        allow_autobranches = False
        start_version = 37

    class Releases(cfg.ReferenceCIConfig.Releases):
        """Releases configuration"""
        allow_robots_to_release_stable = True
        deploy_system = rm_const.DeploySystem.sandbox

        @property
        def releasable_items(self):
            return [
                ri.ReleasableItem(
                    name=rm_conf.RELEASE_RESOURCE,
                    data=ri.SandboxResourceData(rm_conf.RELEASE_RESOURCE, build_ctx_key="arc_svn_url",
                                                attributes={"component_name": self.name}),
                    build_data=ri.BuildData(
                        target="plus/mission_control/package.json",
                    ),
                    deploy_infos=[
                        ri.SandboxInfo(stage=rm_const.ReleaseStatus.stable),
                        ri.SandboxInfo(stage=rm_const.ReleaseStatus.prestable),
                        ri.SandboxInfo(stage=rm_const.ReleaseStatus.testing),
                    ]
                ),
            ]
