import os
import tempfile

from sandbox import sdk2

from sandbox.projects.ydo import get_last_released_resource, execute_cmd, YdoDbManagementExecutable


class YdoBackupInKikimr(sdk2.Task):
    class Parameters(sdk2.Parameters):
        use_stable_resources = sdk2.parameters.Bool(
            "Use stable resources?",
            default=True,
        )
        with use_stable_resources.value[False]:
            ydo_db_executable = sdk2.parameters.Resource(
                "ydo_db",
                resource_type=YdoDbManagementExecutable,
                required=True,
            )

        server = sdk2.parameters.String("YDB endpoint", required=True)

        is_multi_tenant = sdk2.parameters.Bool(
            "Multi Tenant?",
            default=False,
        )
        with is_multi_tenant.value[True]:
            database = sdk2.parameters.String("database", required=True)
            ydb_vault_token = sdk2.parameters.String("Your ydb token name in vault", default="yt-token", required=True)

        database_folder = sdk2.parameters.String("Folder need to backup", required=True)
        backups_folder = sdk2.parameters.String("Backups storage", required=True)

        with sdk2.parameters.Output:
            json_result = sdk2.parameters.String(
                "Backuped tables", required=True
            )

    class Requirements(sdk2.Requirements):
        cores = 1

        class Caches(sdk2.Requirements.Caches):
            pass

    def on_execute(self):
        cmd = [
            str(get_last_released_resource(
                YdoDbManagementExecutable,
                condition=self.Parameters.use_stable_resources,
                default=self.Parameters.ydo_db_executable,
                error_msg="YdoDbManagementExecutable not founded",
            ).path),
            "--server", self.Parameters.server, "--debug"]

        env = os.environ.copy()
        if self.Parameters.is_multi_tenant:
            cmd += ["--database", self.Parameters.database]
            env["YDB_TOKEN"] = sdk2.Vault.data(self.owner, self.Parameters.ydb_vault_token)

        cmd += [
            "backup",
            "--dir", self.Parameters.database_folder,
            "--backups-storage", self.Parameters.backups_folder
        ]

        output = str(tempfile.NamedTemporaryFile().name)
        cmd.extend(["--output", output])

        execute_cmd(cmd, "backup", "backup failed", env=env)

        with open(output) as o:
            self.Parameters.json_result = o.read()
